//=======================================================================
//
// example1.c : Example of use of the library for Integer Coding
// 
// Copyright (C) 2002 Paolo Ferragina (http://www.di.unipi.it/~ferragin)
// Last revision: January 2002
//
// See the COPYRIGHT file for infos on the use of this software
//
//=======================================================================

#include "math.h"
#include "stdlib.h"
#include "stdio.h"
#include "../src/intCodingLib.h"


int main(int argc, char *argv[])
{

  buffer buffer_gamma,buffer_delta,buffer_gammagolomb;
  buffer buffer_golomb,buffer_continuationbit,buffer_test;
  

  unsignedInt i,j,n;
  int written=0;
  unsignedInt k=6;

  // =================   INIT ======================================

  init_buffer(&buffer_gamma);
  init_buffer(&buffer_delta);
  init_buffer(&buffer_gammagolomb);
  init_buffer(&buffer_continuationbit);
  init_buffer(&buffer_golomb);
  init_buffer(&buffer_test);


  // Read the number to be coded
  printf("Input the number to be coded (0 = exit)....\n");
  scanf("%d", &n);

  // Input
  while(n != 0) {
    
    written++; // # of written codes

    // write
    write_gamma_code(&buffer_gamma,n);
    write_delta_code(&buffer_delta,n);
    write_gamma_golomb_code(&buffer_gammagolomb,n,k);
    write_golomb_code(&buffer_golomb,n,k);
	write_continuationbit_code(&buffer_continuationbit,n);

    // Stampa buffers
    printf("\n\nGAMMA: Bytes of the buffer");
    for(i=0; i<BYTES_TO_PRINT; i++){
      printf("\n");
      for(j=0; j<8; j++)
	printf("%d",((((buffer_gamma.pStartBuf)[i]<<j) & 0x80) ? 1 : 0)); }

    printf("\n\nDELTA: Bytes of the buffer");
    for(i=0; i<BYTES_TO_PRINT; i++){
      printf("\n");
      for(j=0; j<8; j++)
	printf("%d",(((buffer_delta.pStartBuf[i]<<j) & 0x80) ? 1 : 0)); }

    printf("\n\nGAMMA GOLOMB: Bytes of the buffer");
    for(i=0; i<BYTES_TO_PRINT; i++){
      printf("\n");
      for(j=0; j<8; j++)
	printf("%d",(((buffer_gammagolomb.pStartBuf[i]<<j) & 0x80) ? 1 : 0)); }

    printf("\n\nGOLOMB: Bytes of the buffer");
    for(i=0; i<BYTES_TO_PRINT; i++){
      printf("\n");
      for(j=0; j<8; j++)
	printf("%d",(((buffer_golomb.pStartBuf[i]<<j) & 0x80) ? 1 : 0)); }

    printf("\n\nCONTINUATION BIT: Bytes of the buffer");
    for(i=0; i<BYTES_TO_PRINT; i++){
      printf("\n");
      for(j=0; j<8; j++)
	printf("%d",(((buffer_continuationbit.pStartBuf[i]<<j) & 0x80) ? 1 : 0)); }

	// Read the number to be coded
    printf("\n\nInput the number to be coded (0 = exit)....\n");
    scanf("%d", &n);
  }

  // Reading what written .... to check

  rewind_buffer(&buffer_gamma);
  rewind_buffer(&buffer_delta);
  rewind_buffer(&buffer_gammagolomb);
  rewind_buffer(&buffer_golomb);
  rewind_buffer(&buffer_continuationbit);

  while (written>0) {
    
    n=read_gamma_code(&buffer_gamma);
    printf("\ngamma:number %d\n", n);
    n=read_delta_code(&buffer_delta);
    printf("delta:number %d\n", n);
    n=read_gamma_golomb_code(&buffer_gammagolomb,k);
    printf("gamma golomb:number %d\n", n);
    n=read_golomb_code(&buffer_golomb,k);
    printf("golomb:number %d\n", n);
    n=read_continuationbit_code(&buffer_continuationbit);
    printf("continuation bit:number %d\n", n);
    written--;
  }


  // =================================================================
  // BRUTE-FORCE TEST

  printf("\n\n\nStarted the brute-force test on 10^7 numbers....\n");

  for(i=1;i<10000000;i++){

    init_buffer(&buffer_test);

    write_gamma_code(&buffer_test,i);
    write_delta_code(&buffer_test,i);
    write_gamma_golomb_code(&buffer_test,i,k);
    write_golomb_code(&buffer_test,i,k);
	write_continuationbit_code(&buffer_test,i);

	rewind_buffer(&buffer_test);

    n=read_gamma_code(&buffer_test); 
    if(n != i){
      printf("error gamma %d",i); exit(1);
    }
    n=read_delta_code(&buffer_test); 
    if(n != i){
      printf("error delta %d",i); exit(1);
    }
    n=read_gamma_golomb_code(&buffer_test,k); 
    if(n != i){
      printf("error gamma golomb %d",i); exit(1);
	}
    n=read_golomb_code(&buffer_test,k); 
    if(n != i){
      printf("error golomb %d",i); exit(1);
	}
    n=read_continuationbit_code(&buffer_test); 
    if(n != i){
      printf("error continuation bit %d",i); exit(1);
    }

	delete_buffer(&buffer_test);
  }

  printf("\n\nPress 0+Enter to exit.\n");
  scanf("%d", &n);
}

