//=======================================================================
//
// example2.c : Example of use of the library for Integer Coding
// 
// Copyright (C) 2002 Paolo Ferragina (http://www.di.unipi.it/~ferragin)
// Last revision: January 2002
//
// See the COPYRIGHT file for infos on the use of this software
//
//=======================================================================



#include "math.h"
#include "stdlib.h"
#include "stdio.h"
#include "../src/intCodingLib.h"


int main(int argc, char *argv[])
{

  buffer mio_buffer;
  

  unsignedInt i,j,n,x;
  unsignedInt k=6;

  // Read the number to be coded
  printf("Input the number to random integers to be coded....\n");
  scanf("%d", &x);

  // =================   INIT ======================================

  init_buffer(&mio_buffer);
  srand(17);


  printf("\n\n\nStarted the test....\n");

  for(i=1;i<x;i++){

    n=1+(int) (10.0*rand()/(RAND_MAX+1.0));

    write_gamma_code(&mio_buffer,n);
    write_delta_code(&mio_buffer,n);
    write_gamma_golomb_code(&mio_buffer,n,k);
    write_golomb_code(&mio_buffer,n,k);
    write_continuationbit_code(&mio_buffer,n);

  }

  printf("Buffer size of %d bits\n",get_buffer_size(&mio_buffer));
  
  rewind_buffer(&mio_buffer);
  srand(17);

  printf("Read the buffer and check for consistency\n");

  for(i=1;i<x;i++){

    n=1+(int) (10.0*rand()/(RAND_MAX+1.0));

    j=read_gamma_code(&mio_buffer);
    if(j!=n){
      printf("error on GAMMA read=%d written=%d\n",j,n);exit(1);
    }

    j=read_delta_code(&mio_buffer);
    if(j!=n){
      printf("error on DELTA read=%d written=%d\n",j,n);exit(1);
    }


    read_gamma_golomb_code(&mio_buffer,k);
    if(j!=n){
      printf("error on GGOLOMB read=%d written=%d\n",j,n);exit(1);
    }

    read_golomb_code(&mio_buffer,k);
    if(j!=n){
      printf("error on GOLOMB read=%d written=%d\n",j,n);exit(1);
    }

    read_continuationbit_code(&mio_buffer);
    if(j!=n){
      printf("error on CONTINUATION read=%d written=%d\n",j,n);exit(1);
    }

  }

  delete_buffer(&mio_buffer);

  printf("\n....OK TEST\n\n");
}







